@@ -9,30 +9,6 @@ use Encode;
 
 package IkiWiki;
 
-sub printheader ($) { #{{{
-	my $session=shift;
-	
-	if ($config{sslcookie}) {
-		print $session->header(-charset => 'utf-8',
-			-cookie => $session->cookie(-secure => 1));
-	} else {
-		print $session->header(-charset => 'utf-8');
-	}
-
-} #}}}
-
-sub redirect ($$) { #{{{
-	my $q=shift;
-	my $url=shift;
-	if (! $config{w3mmode}) {
-		print $q->redirect($url);
-	}
-	else {
-		print "Content-type: text/plain\n";
-		print "W3m-control: GOTO $url\n\n";
-	}
-} #}}}
-
 sub page_locked ($$;$) { #{{{
 	my $page=shift;
 	my $session=shift;
@@ -43,70 +19,38 @@ sub page_locked ($$;$) { #{{{
 
 	foreach my $admin (@{$config{adminuser}}) {
 		my $locked_pages=userinfo_get($admin, "locked_pages");
-		if (pagespec_match($page, userinfo_get($admin, "locked_pages"))) {
+		if (globlist_match($page, userinfo_get($admin, "locked_pages"))) {
 			return 1 if $nonfatal;
 			error(htmllink("", "", $page, 1)." is locked by ".
-			      userlink($admin)." and cannot be edited.");
+			      htmllink("", "", $admin, 1)." and cannot be edited.");
 		}
 	}
 
 	return 0;
 } #}}}
 
-sub decode_form_utf8 ($) { #{{{
-	my $form = shift;
-	foreach my $f ($form->field) {
-		next if Encode::is_utf8(scalar $form->field($f));
-		$form->field(name  => $f,
-			     value => decode_utf8($form->field($f)),
-			     force => 1,
-			    );
-	}
-} #}}}
-
 sub cgi_recentchanges ($) { #{{{
 	my $q=shift;
 	
+	unlockwiki();
+
 	# Optimisation: building recentchanges means calculating lots of
 	# links. Memoizing htmllink speeds it up a lot (can't be memoized
 	# during page builds as the return values may change, but they
 	# won't here.)
 	eval q{use Memoize};
-	error($@) if $@;
 	memoize("htmllink");
 
-	eval q{use Time::Duration};
-	error($@) if $@;
-
-	my $changelog=[rcs_recentchanges(100)];
-	foreach my $change (@$changelog) {
-		$change->{when} = concise(ago($change->{when}));
-
-		$change->{user} = userlink($change->{user});
-
-		my $is_excess = exists $change->{pages}[10]; # limit pages to first 10
-		delete @{$change->{pages}}[10 .. @{$change->{pages}}] if $is_excess;
-		$change->{pages} = [
-			map {
-				$_->{link} = htmllink("", "", $_->{page}, 1);
-				$_;
-			} @{$change->{pages}}
-		];
-		push @{$change->{pages}}, { link => '...' } if $is_excess;
-	}
-
 	my $template=template("recentchanges.tmpl"); 
 	$template->param(
 		title => "RecentChanges",
 		indexlink => indexlink(),
 		wikiname => $config{wikiname},
-		changelog => $changelog,
-		baseurl => baseurl(),
+		changelog => [rcs_recentchanges(100)],
+		styleurl => styleurl(),
+		baseurl => "$config{url}/",
 	);
-	run_hooks(pagetemplate => sub {
-		shift->(page => "", destpage => "", template => $template);
-	});
-	print $q->header(-charset => 'utf-8'), $template->output;
+	print $q->header(-charset=>'utf-8'), $template->output;
 } #}}}
 
 sub cgi_signin ($$) { #{{{
@@ -114,12 +58,18 @@ sub cgi_signin ($$) { #{{{
 	my $session=shift;
 
 	eval q{use CGI::FormBuilder};
-	error($@) if $@;
 	my $form = CGI::FormBuilder->new(
 		title => "signin",
+		fields => [qw(do title page subpage from name password confirm_password email)],
 		header => 1,
 		charset => "utf-8",
 		method => 'POST',
+		validate => {
+			confirm_password => {
+				perl => q{eq $form->field("password")},
+			},
+			email => 'EMAIL',
+		},
 		required => 'NONE',
 		javascript => 0,
 		params => $q,
@@ -127,52 +77,144 @@ sub cgi_signin ($$) { #{{{
 		header => 0,
 		template => (-e "$config{templatedir}/signin.tmpl" ?
 			     {template_params("signin.tmpl")} : ""),
-		stylesheet => baseurl()."style.css",
+		stylesheet => styleurl(),
 	);
-	my $buttons=["Login"];
 	
+	$form->field(name => "name", required => 0);
 	$form->field(name => "do", type => "hidden");
-	
-	if ($q->param("do") ne "signin" && !$form->submitted) {
+	$form->field(name => "page", type => "hidden");
+	$form->field(name => "title", type => "hidden");
+	$form->field(name => "from", type => "hidden");
+	$form->field(name => "subpage", type => "hidden");
+	$form->field(name => "password", type => "password", required => 0);
+	$form->field(name => "confirm_password", type => "password", required => 0);
+	$form->field(name => "email", required => 0);
+	if ($q->param("do") ne "signin") {
 		$form->text("You need to log in first.");
 	}
 	
-	run_hooks(formbuilder_setup => sub {
-		shift->(form => $form, cgi => $q, session => $session);
-	});
+	if ($form->submitted) {
+		# Set required fields based on how form was submitted.
+		my %required=(
+			"Login" => [qw(name password)],
+			"Register" => [qw(name password confirm_password email)],
+			"Mail Password" => [qw(name)],
+		);
+		foreach my $opt (@{$required{$form->submitted}}) {
+			$form->field(name => $opt, required => 1);
+		}
 	
-	decode_form_utf8($form);
-
-	if (exists $hooks{formbuilder}) {
-		run_hooks(formbuilder => sub {
-			shift->(form => $form, cgi => $q, session => $session,
-				buttons => $buttons);
-		});
-	}
-	else {
-		if ($form->submitted) {
-			$form->validate;
+		# Validate password differently depending on how
+		# form was submitted.
+		if ($form->submitted eq 'Login') {
+			$form->field(
+				name => "password",
+				validate => sub {
+					length $form->field("name") &&
+					shift eq userinfo_get($form->field("name"), 'password');
+				},
+			);
+			$form->field(name => "name", validate => '/^\w+$/');
+		}
+		else {
+			$form->field(name => "password", validate => 'VALUE');
+		}
+		# And make sure the entered name exists when logging
+		# in or sending email, and does not when registering.
+		if ($form->submitted eq 'Register') {
+			$form->field(
+				name => "name",
+				validate => sub {
+					my $name=shift;
+					length $name &&
+					$name=~/$config{wiki_file_regexp}/ &&
+					! userinfo_get($name, "regdate");
+				},
+			);
+		}
+		else {
+			$form->field(
+				name => "name",
+				validate => sub {
+					my $name=shift;
+					length $name &&
+					userinfo_get($name, "regdate");
+				},
+			);
 		}
-		printheader($session);
-		print misctemplate($form->title, $form->render(submit => $buttons));
-	}
-} #}}}
-
-sub cgi_postsignin ($$) { #{{{
-	my $q=shift;
-	my $session=shift;
-
-	# Continue with whatever was being done before the signin process.
-	if (defined $q->param("do") && $q->param("do") ne "signin" &&
-	    defined $session->param("postsignin")) {
-		my $postsignin=CGI->new($session->param("postsignin"));
-		$session->clear("postsignin");
-		cgi($postsignin, $session);
-		cgi_savesession($session);
-		exit;
 	}
 	else {
-		redirect($q, $config{url});
+		# First time settings.
+		$form->field(name => "name", comment => "use FirstnameLastName");
+		$form->field(name => "confirm_password", comment => "(only needed");
+		$form->field(name => "email",            comment => "for registration)");
+		if ($session->param("name")) {
+			$form->field(name => "name", value => $session->param("name"));
+		}
+	}
+
+	if ($form->submitted && $form->validate) {
+		if ($form->submitted eq 'Login') {
+			$session->param("name", $form->field("name"));
+			if (defined $form->field("do") && 
+			    $form->field("do") ne 'signin') {
+				print $q->redirect(cgiurl(
+					do => $form->field("do"),
+					page => $form->field("page"),
+					title => $form->field("title"),
+					subpage => $form->field("subpage"),
+					from => $form->field("from"),
+				));
+			}
+			else {
+				print $q->redirect($config{url});
+			}
+		}
+		elsif ($form->submitted eq 'Register') {
+			my $user_name=$form->field('name');
+			if (userinfo_setall($user_name, {
+				           'email' => $form->field('email'),
+				           'password' => $form->field('password'),
+				           'regdate' => time
+				         })) {
+				$form->field(name => "confirm_password", type => "hidden");
+				$form->field(name => "email", type => "hidden");
+				$form->text("Registration successful. Now you can Login.");
+				print $session->header(-charset=>'utf-8');
+				print misctemplate($form->title, $form->render(submit => ["Login"]));
+			}
+			else {
+				error("Error saving registration.");
+			}
+		}
+		elsif ($form->submitted eq 'Mail Password') {
+			my $user_name=$form->field("name");
+			my $template=template("passwordmail.tmpl");
+			$template->param(
+				user_name => $user_name,
+				user_password => userinfo_get($user_name, "password"),
+				wikiurl => $config{url},
+				wikiname => $config{wikiname},
+				REMOTE_ADDR => $ENV{REMOTE_ADDR},
+			);
+			
+			eval q{use Mail::Sendmail};
+			sendmail(
+				To => userinfo_get($user_name, "email"),
+				From => "$config{wikiname} admin <$config{adminemail}>",
+				Subject => "$config{wikiname} information",
+				Message => $template->output,
+			) or error("Failed to send mail");
+			
+			$form->text("Your password has been emailed to you.");
+			$form->field(name => "name", required => 0);
+			print $session->header(-charset=>'utf-8');
+			print misctemplate($form->title, $form->render(submit => ["Login", "Register", "Mail Password"]));
+		}
+	}
+	else {
+		print $session->header(-charset=>'utf-8');
+		print misctemplate($form->title, $form->render(submit => ["Login", "Register", "Mail Password"]));
 	}
 } #}}}
 
@@ -181,13 +223,17 @@ sub cgi_prefs ($$) { #{{{
 	my $session=shift;
 
 	eval q{use CGI::FormBuilder};
-	error($@) if $@;
 	my $form = CGI::FormBuilder->new(
 		title => "preferences",
+		fields => [qw(do name password confirm_password email 
+		              subscriptions locked_pages)],
 		header => 0,
 		charset => "utf-8",
 		method => 'POST',
 		validate => {
+			confirm_password => {
+				perl => q{eq $form->field("password")},
+			},
 			email => 'EMAIL',
 		},
 		required => 'NONE',
@@ -196,28 +242,25 @@ sub cgi_prefs ($$) { #{{{
 		action => $config{cgiurl},
 		template => (-e "$config{templatedir}/prefs.tmpl" ?
 			     {template_params("prefs.tmpl")} : ""),
-		stylesheet => baseurl()."style.css",
+		stylesheet => styleurl(),
 	);
-	my $buttons=["Save Preferences", "Logout", "Cancel"];
-
-	run_hooks(formbuilder_setup => sub {
-		shift->(form => $form, cgi => $q, session => $session);
-	});
-	
-	$form->field(name => "do", type => "hidden");
-	$form->field(name => "email", size => 50);
-	$form->field(name => "subscriptions", size => 50,
-		comment => "(".htmllink("", "", "PageSpec", 1).")");
-	$form->field(name => "locked_pages", size => 50,
-		comment => "(".htmllink("", "", "PageSpec", 1).")");
-	$form->field(name => "banned_users", size => 50);
+	my @buttons=("Save Preferences", "Logout", "Cancel");
 	
 	my $user_name=$session->param("name");
+	$form->field(name => "do", type => "hidden");
+	$form->field(name => "name", disabled => 1,
+		value => $user_name, force => 1);
+	$form->field(name => "password", type => "password");
+	$form->field(name => "confirm_password", type => "password");
+	$form->field(name => "subscriptions", size => 50,
+		comment => "(".htmllink("", "", "GlobList", 1).")");
+	$form->field(name => "locked_pages", size => 50,
+		comment => "(".htmllink("", "", "GlobList", 1).")");
+	
 	if (! is_admin($user_name)) {
 		$form->field(name => "locked_pages", type => "hidden");
-		$form->field(name => "banned_users", type => "hidden");
 	}
-
+	
 	if (! $form->submitted) {
 		$form->field(name => "email", force => 1,
 			value => userinfo_get($user_name, "email"));
@@ -225,59 +268,37 @@ sub cgi_prefs ($$) { #{{{
 			value => userinfo_get($user_name, "subscriptions"));
 		$form->field(name => "locked_pages", force => 1,
 			value => userinfo_get($user_name, "locked_pages"));
-		if (is_admin($user_name)) {
-			$form->field(name => "banned_users", force => 1,
-				value => join(" ", get_banned_users()));
-		}
 	}
 	
-	decode_form_utf8($form);
-	
 	if ($form->submitted eq 'Logout') {
 		$session->delete();
-		redirect($q, $config{url});
+		print $q->redirect($config{url});
 		return;
 	}
 	elsif ($form->submitted eq 'Cancel') {
-		redirect($q, $config{url});
+		print $q->redirect($config{url});
 		return;
 	}
 	elsif ($form->submitted eq "Save Preferences" && $form->validate) {
-		foreach my $field (qw(email subscriptions locked_pages)) {
-			if (defined $form->field($field) && length $form->field($field)) {
+		foreach my $field (qw(password email subscriptions locked_pages)) {
+			if (length $form->field($field)) {
 				userinfo_set($user_name, $field, $form->field($field)) || error("failed to set $field");
 			}
 		}
-		if (is_admin($user_name)) {
-			set_banned_users(grep { ! is_admin($_) }
-					split(' ', $form->field("banned_users")));
-		}
 		$form->text("Preferences saved.");
 	}
 	
-	if (exists $hooks{formbuilder}) {
-		run_hooks(formbuilder => sub {
-			shift->(form => $form, cgi => $q, session => $session,
-				buttons => $buttons);
-		});
-	}
-	else {
-		printheader($session);
-		print misctemplate($form->title, $form->render(submit => $buttons));
-	}
+	print $session->header(-charset=>'utf-8');
+	print misctemplate($form->title, $form->render(submit => \@buttons));
 } #}}}
 
 sub cgi_editpage ($$) { #{{{
 	my $q=shift;
 	my $session=shift;
 
-	my @fields=qw(do rcsinfo subpage from page type editcontent comments);
-	my @buttons=("Save Page", "Preview", "Cancel");
-	
 	eval q{use CGI::FormBuilder};
-	error($@) if $@;
 	my $form = CGI::FormBuilder->new(
-		fields => \@fields,
+		fields => [qw(do rcsinfo subpage from page editcontent comments)],
 		header => 1,
 		charset => "utf-8",
 		method => 'POST',
@@ -289,49 +310,27 @@ sub cgi_editpage ($$) { #{{{
 		params => $q,
 		action => $config{cgiurl},
 		table => 0,
-		template => (-e "$config{templatedir}/editpage.tmpl" ?
-				{template_params("editpage.tmpl")} : ""),
+		template => {template_params("editpage.tmpl")},
 	);
+	my @buttons=("Save Page", "Preview", "Cancel");
 	
-	run_hooks(formbuilder_setup => sub {
-		shift->(form => $form, cgi => $q, session => $session);
-	});
-	
-	decode_form_utf8($form);
-	
-	# This untaint is safe because titlepage removes any problematic
+	# This untaint is safe because titlepage removes any problimatic
 	# characters.
-	my ($page)=$form->field('page');
-	$page=titlepage(possibly_foolish_untaint($page));
+	my ($page)=Encode::decode_utf8($form->param('page'));
+	$page=titlepage(possibly_foolish_untaint(lc($page)));
 	if (! defined $page || ! length $page ||
 	    $page=~/$config{wiki_file_prune_regexp}/ || $page=~/^\//) {
 		error("bad page name");
 	}
-	
-	my $from;
-	if (defined $form->field('from')) {
-		($from)=$form->field('from')=~/$config{wiki_file_regexp}/;
-	}
+	$page=lc($page);
 	
 	my $file;
-	my $type;
-	if (exists $pagesources{$page}) {
-		$file=$pagesources{$page};
-		$type=pagetype($file);
+	if (exists $pagesources{lc($page)}) {
+		$file=$pagesources{lc($page)};
 	}
 	else {
-		$type=$form->param('type');
-		if (defined $type && length $type && $hooks{htmlize}{$type}) {
-			$type=possibly_foolish_untaint($type);
-		}
-		elsif (defined $from) {
-			# favor the type of linking page
-			$type=pagetype($pagesources{$from});
-		}
-		$type=$config{default_pageext} unless defined $type;
-		$file=$page.".".$type;
+		$file=$page.".".$config{default_pageext};
 	}
-
 	my $newfile=0;
 	if (! -e "$config{srcdir}/$file") {
 		$newfile=1;
@@ -341,8 +340,7 @@ sub cgi_editpage ($$) { #{{{
 	$form->field(name => "from", type => 'hidden');
 	$form->field(name => "rcsinfo", type => 'hidden');
 	$form->field(name => "subpage", type => 'hidden');
-	$form->field(name => "page", value => $page, force => 1);
-	$form->field(name => "type", value => $type, force => 1);
+	$form->field(name => "page", value => "$page", force => 1);
 	$form->field(name => "comments", type => "text", size => 80);
 	$form->field(name => "editcontent", type => "textarea", rows => 20,
 		cols => 80);
@@ -350,37 +348,30 @@ sub cgi_editpage ($$) { #{{{
 	$form->tmpl_param("indexlink", indexlink());
 	$form->tmpl_param("helponformattinglink",
 		htmllink("", "", "HelpOnFormatting", 1));
-	$form->tmpl_param("baseurl", baseurl());
+	$form->tmpl_param("styleurl", styleurl());
+	$form->tmpl_param("baseurl", "$config{url}/");
 	if (! $form->submitted) {
 		$form->field(name => "rcsinfo", value => rcs_prepedit($file),
 			force => 1);
 	}
 	
 	if ($form->submitted eq "Cancel") {
-		if ($newfile && defined $from) {
-			redirect($q, "$config{url}/".htmlpage($from));
-		}
-		elsif ($newfile) {
-			redirect($q, $config{url});
-		}
-		else {
-			redirect($q, "$config{url}/".htmlpage($page));
-		}
+		print $q->redirect("$config{url}/".htmlpage($page));
 		return;
 	}
 	elsif ($form->submitted eq "Preview") {
-		my $content=$form->field('editcontent');
-		my $comments=$form->field('comments');
+		require IkiWiki::Render;
+		# Apparently FormBuilder doesn't not treat input as
+		# utf-8, so decode from it.
+		my $content = Encode::decode_utf8($form->field('editcontent'));
+		my $comments = Encode::decode_utf8($form->field('comments'));
 		$form->field(name => "editcontent",
 				value => $content, force => 1);
 		$form->field(name => "comments",
 				value => $comments, force => 1);
-		$config{rss}=$config{atom}=0; # avoid preview writing a feed!
 		$form->tmpl_param("page_preview",
-			htmlize($page, $type,
-			linkify($page, "",
-			preprocess($page, $page,
-			filter($page, $content)))));
+			htmlize(pagetype($file),
+				linkify($page, $page, $content)));
 	}
 	else {
 		$form->tmpl_param("page_preview", "");
@@ -392,8 +383,9 @@ sub cgi_editpage ($$) { #{{{
 		if ($form->field("do") eq "create") {
 			my @page_locs;
 			my $best_loc;
+			my ($from)=$form->param('from')=~/$config{wiki_file_regexp}/;
 			if (! defined $from || ! length $from ||
-			    $from ne $form->field('from') ||
+			    $from ne $form->param('from') ||
 			    $from=~/$config{wiki_file_prune_regexp}/ ||
 			    $from=~/^\// ||
 			    $form->submitted eq "Preview") {
@@ -403,7 +395,7 @@ sub cgi_editpage ($$) { #{{{
 				my $dir=$from."/";
 				$dir=~s![^/]+/+$!!;
 				
-				if ((defined $form->field('subpage') && length $form->field('subpage')) ||
+				if ((defined $form->param('subpage') && length $form->param('subpage')) ||
 				    $page eq 'discussion') {
 					$best_loc="$from/$page";
 				}
@@ -420,27 +412,20 @@ sub cgi_editpage ($$) { #{{{
 			}
 
 			@page_locs = grep {
-				! exists $pagecase{lc $_} &&
+				! exists $pagesources{lc($_)} &&
 				! page_locked($_, $session, 1)
 			} @page_locs;
 			
 			if (! @page_locs) {
 				# hmm, someone else made the page in the
 				# meantime?
-				redirect($q, "$config{url}/".htmlpage($page));
+				print $q->redirect("$config{url}/".htmlpage($page));
 				return;
 			}
-			
-			my @page_types;
-			if (exists $hooks{htmlize}) {
-				@page_types=keys %{$hooks{htmlize}};
-			}
-			
+				
 			$form->tmpl_param("page_select", 1);
 			$form->field(name => "page", type => 'select',
 				options => \@page_locs, value => $best_loc);
-			$form->field(name => "type", type => 'select',
-				options => \@page_types);
 			$form->title("creating ".pagetitle($page));
 		}
 		elsif ($form->field("do") eq "edit") {
@@ -448,8 +433,8 @@ sub cgi_editpage ($$) { #{{{
 			if (! defined $form->field('editcontent') || 
 			    ! length $form->field('editcontent')) {
 				my $content="";
-				if (exists $pagesources{$page}) {
-					$content=readfile(srcfile($pagesources{$page}));
+				if (exists $pagesources{lc($page)}) {
+					$content=readfile(srcfile($pagesources{lc($page)}));
 					$content=~s/\n/\r\n/g;
 				}
 				$form->field(name => "editcontent", value => $content,
@@ -457,7 +442,6 @@ sub cgi_editpage ($$) { #{{{
 			}
 			$form->tmpl_param("page_select", 0);
 			$form->field(name => "page", type => 'hidden');
-			$form->field(name => "type", type => 'hidden');
 			$form->title("editing ".pagetitle($page));
 		}
 		
@@ -467,19 +451,27 @@ sub cgi_editpage ($$) { #{{{
 		# save page
 		page_locked($page, $session);
 		
-		my $content=$form->field('editcontent');
+		# Decode utf-8 since FormBuilder does not
+		my $content=Encode::decode_utf8($form->field('editcontent'));
 
 		$content=~s/\r\n/\n/g;
 		$content=~s/\r/\n/g;
 		writefile($file, $config{srcdir}, $content);
 		
+		my $message="web commit ";
+		if (defined $session->param("name") && 
+		    length $session->param("name")) {
+			$message.="by ".$session->param("name");
+		}
+		else {
+			$message.="from $ENV{REMOTE_ADDR}";
+		}
+		if (defined $form->field('comments') &&
+		    length $form->field('comments')) {
+			$message.=": ".Encode::decode_utf8($form->field('comments'));
+		}
+		
 		if ($config{rcs}) {
-			my $message="";
-			if (defined $form->field('comments') &&
-			    length $form->field('comments')) {
-				$message=$form->field('comments');
-			}
-			
 			if ($newfile) {
 				rcs_add($file);
 			}
@@ -488,19 +480,17 @@ sub cgi_editpage ($$) { #{{{
 			# presumably the commit will trigger an update
 			# of the wiki
 			my $conflict=rcs_commit($file, $message,
-				$form->field("rcsinfo"),
-				$session->param("name"), $ENV{REMOTE_ADDR});
+				$form->field("rcsinfo"));
 		
 			if (defined $conflict) {
 				$form->field(name => "rcsinfo", value => rcs_prepedit($file),
 					force => 1);
 				$form->tmpl_param("page_conflict", 1);
 				$form->field("editcontent", value => $conflict, force => 1);
-				$form->field(name => "comments", value => $form->field('comments'), force => 1);
+				$form->field(name => "comments", value => Encode::decode_utf8($form->field('comments')), force => 1);
 				$form->field("do", "edit)");
 				$form->tmpl_param("page_select", 0);
 				$form->field(name => "page", type => 'hidden');
-				$form->field(name => "type", type => 'hidden');
 				$form->title("editing $page");
 				print $form->render(submit => \@buttons);
 				return;
@@ -514,55 +504,25 @@ sub cgi_editpage ($$) { #{{{
 		
 		# The trailing question mark tries to avoid broken
 		# caches and get the most recent version of the page.
-		redirect($q, "$config{url}/".htmlpage($page)."?updated");
+		print $q->redirect("$config{url}/".htmlpage($page)."?updated");
 	}
 } #}}}
 
-sub cgi_getsession ($) { #{{{
-	my $q=shift;
-
+sub cgi () { #{{{
+	eval q{use CGI};
 	eval q{use CGI::Session};
-	CGI::Session->name("ikiwiki_session_".encode_utf8($config{wikiname}));
 	
-	my $oldmask=umask(077);
-	my $session = CGI::Session->new("driver:DB_File", $q,
-		{ FileName => "$config{wikistatedir}/sessions.db" });
-	umask($oldmask);
-
-	return $session;
-} #}}}
-
-sub cgi_savesession ($) { #{{{
-	my $session=shift;
-
-	# Force session flush with safe umask.
-	my $oldmask=umask(077);
-	$session->flush;
-	umask($oldmask);
-}
-
-sub cgi (;$$) { #{{{
-	my $q=shift;
-	my $session=shift;
-
-	if (! $q) {
-		eval q{use CGI};
-		error($@) if $@;
+	my $q=CGI->new;
 	
-		$q=CGI->new;
-	
-		run_hooks(cgi => sub { shift->($q) });
+	if (exists $hooks{cgi}) {
+		foreach my $id (keys %{$hooks{cgi}}) {
+			$hooks{cgi}{$id}{call}->($q);
+		}
 	}
-
+	
 	my $do=$q->param('do');
 	if (! defined $do || ! length $do) {
-		my $error = $q->cgi_error;
-		if ($error) {
-			error("Request not processed: $error");
-		}
-		else {
-			error("\"do\" parameter missing");
-		}
+		error("\"do\" parameter missing");
 	}
 	
 	# Things that do not need a session.
@@ -573,55 +533,27 @@ sub cgi (;$$) { #{{{
 	elsif ($do eq 'hyperestraier') {
 		cgi_hyperestraier();
 	}
-
-	# Need to lock the wiki before getting a session.
-	lockwiki();
 	
-	if (! $session) {
-		$session=cgi_getsession($q);
-	}
+	CGI::Session->name("ikiwiki_session_$config{wikiname}");
+	
+	my $oldmask=umask(077);
+	my $session = CGI::Session->new("driver:DB_File", $q,
+		{ FileName => "$config{wikistatedir}/sessions.db" });
+	umask($oldmask);
 	
-	# Auth hooks can sign a user in.
-	if ($do ne 'signin' && ! defined $session->param("name")) {
-		run_hooks(auth => sub {
-			shift->($q, $session)
-		});
-		if (defined $session->param("name")) {
-			# Make sure whatever user was authed is in the
-			# userinfo db.
-			if (! userinfo_get($session->param("name"), "regdate")) {
-				userinfo_setall($session->param("name"), {
-					email => "",
-					password => "",
-					regdate => time,
-				});
-			}
-		}
-	}
-
 	# Everything below this point needs the user to be signed in.
-	if (((! $config{anonok} || $do eq 'prefs') &&
+	if ((! $config{anonok} &&
 	     (! defined $session->param("name") ||
-	     ! userinfo_get($session->param("name"), "regdate")))
-            || $do eq 'signin') {
-	    	if ($do ne 'signin' && ! defined $session->param("postsignin")) {
-			$session->param(postsignin => $ENV{QUERY_STRING});
-		}
+	     ! userinfo_get($session->param("name"), "regdate"))) || $do eq 'signin') {
 		cgi_signin($q, $session);
-		cgi_savesession($session);
+	
+		# Force session flush with safe umask.
+		my $oldmask=umask(077);
+		$session->flush;
+		umask($oldmask);
+		
 		return;
 	}
-	elsif (defined $session->param("postsignin")) {
-		cgi_postsignin($q, $session);
-	}
-
-	if (defined $session->param("name") && userinfo_get($session->param("name"), "banned")) {
-		print $q->header(-status => "403 Forbidden");
-		$session->delete();
-		print "You are banned.";
-		cgi_savesession($session);
-		exit;
-	}
 	
 	if ($do eq 'create' || $do eq 'edit') {
 		cgi_editpage($q, $session);
@@ -630,11 +562,11 @@ sub cgi (;$$) { #{{{
 		cgi_prefs($q, $session);
 	}
 	elsif ($do eq 'blog') {
-		my $page=titlepage(decode_utf8($q->param('title')));
+		my $page=titlepage(lc($q->param('title')));
 		# if the page already exists, munge it to be unique
 		my $from=$q->param('from');
 		my $add="";
-		while (exists $pagecase{lc "$from/$page$add"}) {
+		while (exists $oldpagemtime{"$from/$page$add"}) {
 			$add=1 unless length $add;
 			$add++;
 		}
@@ -648,26 +580,4 @@ sub cgi (;$$) { #{{{
 	}
 } #}}}
 
-sub userlink ($) { #{{{
-	my $user=shift;
-
-	eval q{use CGI 'escapeHTML'};
-	error($@) if $@;
-	if ($user =~ m!^https?://! &&
-	    eval q{use Net::OpenID::VerifiedIdentity; 1} && !$@) {
-		# Munge user-urls, as used by eg, OpenID.
-		my $oid=Net::OpenID::VerifiedIdentity->new(identity => $user);
-		my $display=$oid->display;
-		# Convert "user.somehost.com" to "user [somehost.com]".
-		if ($display !~ /\[/) {
-			$display=~s/^(.*?)\.([^.]+\.[a-z]+)$/$1 [$2]/;
-		}
-		$display=~s!^https?://!!; # make sure this is removed
-		return "<a href=\"$user\">".escapeHTML($display)."</a>";
-	}
-	else {
-		return htmllink("", "", escapeHTML($user), 1);
-	}
-} #}}}
-
 1
