@@ -7,7 +7,8 @@ use strict;
 use IkiWiki;
 
 sub import { #{{{
-	hook(type => "preprocess", id => "brokenlinks", call => \&preprocess);
+	IkiWiki::hook(type => "preprocess", id => "brokenlinks",
+		call => \&preprocess);
 } # }}}
 
 sub preprocess (@) { #{{{
@@ -16,19 +17,19 @@ sub preprocess (@) { #{{{
 	
 	# Needs to update whenever a page is added or removed, so
 	# register a dependency.
-	add_depends($params{page}, $params{pages});
+	IkiWiki::add_depends($params{page}, $params{pages});
 	
 	my @broken;
-	foreach my $page (keys %links) {
-		if (pagespec_match($page, $params{pages})) {
-			foreach my $link (@{$links{$page}}) {
-				next if $link =~ /.*\/discussion/i && $config{discussion};
-				my $bestlink=bestlink($page, $link);
+	foreach my $page (%IkiWiki::links) {
+		if (IkiWiki::globlist_match($page, $params{pages})) {
+			foreach my $link (@{$IkiWiki::links{$page}}) {
+				next if $link =~ /.*\/discussion/i && $IkiWiki::config{discussion};
+				my $bestlink=IkiWiki::bestlink($page, $link);
 				next if length $bestlink;
 				push @broken,
-					htmllink($page, $params{destpage}, $link, 1).
-					" from ".
-					htmllink($params{page}, $params{destpage}, $page, 1);
+					IkiWiki::htmllink($page, $page, $link, 1).
+					" in ".
+					IkiWiki::htmllink($params{page}, $params{page}, $page, 1);
 			}
 		}
 	}
