@@ -1,32 +1,13 @@
 #!/usr/bin/perl
+# For subversion support.
 
 use warnings;
 use strict;
 use IkiWiki;
-use POSIX qw(setlocale LC_CTYPE);
 
 package IkiWiki;
 		
-# svn needs LC_CTYPE set to a UTF-8 locale, so try to find one. Any will do.
-sub find_lc_ctype() {
-	my $current = setlocale(LC_CTYPE());
-	return $current if $current =~ m/UTF-?8$/i;
-
-	# Make some obvious attempts to avoid calling `locale -a`
-	foreach my $locale ("$current.UTF-8", "en_US.UTF-8", "en_GB.UTF-8") {
-		return $locale if setlocale(LC_CTYPE(), $locale);
-	}
-
-	# Try to get all available locales and pick the first UTF-8 one found.
-	if (my @locale = grep(/UTF-?8$/i, `locale -a`)) {
-		chomp @locale;
-		return $locale[0] if setlocale(LC_CTYPE(), $locale[0]);
-	}
-
-	# fallback to the current locale
-	return $current;
-} # }}}
-$ENV{LC_CTYPE} = $ENV{LC_CTYPE} || find_lc_ctype();
+my $svn_webcommit=qr/^web commit by (\w+):?(.*)/;
 
 sub svn_info ($$) { #{{{
 	my $field=shift;
@@ -60,22 +41,13 @@ sub rcs_prepedit ($) { #{{{
 	}
 } #}}}
 
-sub rcs_commit ($$$;$$) { #{{{
+sub rcs_commit ($$$) { #{{{
 	# Tries to commit the page; returns undef on _success_ and
 	# a version of the page with the rcs's conflict markers on failure.
 	# The file is relative to the srcdir.
 	my $file=shift;
 	my $message=shift;
 	my $rcstoken=shift;
-	my $user=shift;
-	my $ipaddr=shift;
-
-	if (defined $user) {
-		$message="web commit by $user".(length $message ? ": $message" : "");
-	}
-	elsif (defined $ipaddr) {
-		$message="web commit from $ipaddr".(length $message ? ": $message" : "");
-	}
 
 	if (-d "$config{srcdir}/.svn") {
 		# Check to see if the page has been changed by someone
@@ -129,27 +101,13 @@ sub rcs_recentchanges ($) { #{{{
 	
 	return unless -d "$config{srcdir}/.svn";
 
-	eval q{
-		use Date::Parse;
-		use XML::SAX;
-		use XML::Simple;
-	};
-	error($@) if $@;
-
-	# avoid using XML::SAX::PurePerl, it's buggy with UTF-8 data
-	my @parsers = map { ${$_}{Name} } @{XML::SAX->parsers()};
-	do {
-		$XML::Simple::PREFERRED_PARSER = pop @parsers;
-	} until $XML::Simple::PREFERRED_PARSER ne 'XML::SAX::PurePerl';
-
-	# --limit is only supported on Subversion 1.2.0+
-	my $svn_version=`svn --version -q`;
-	my $svn_limit='';
-	$svn_limit="--limit $num"
-		if $svn_version =~ /\d\.(\d)\.\d/ && $1 >= 2;
-
+	eval q{use CGI 'escapeHTML'};
+	eval q{use Date::Parse};
+	eval q{use Time::Duration};
+	eval q{use XML::Simple};
+	
 	my $svn_url=svn_info("URL", $config{srcdir});
-	my $xml = XMLin(scalar `svn $svn_limit --xml -v log '$svn_url'`,
+	my $xml = XMLin(scalar `svn --xml -v log '$svn_url'`,
 		ForceArray => [ 'logentry', 'path' ],
 		GroupTags => { paths => 'path' },
 		KeyAttr => { path => 'content' },
@@ -160,17 +118,20 @@ sub rcs_recentchanges ($) { #{{{
 		my $rev = $logentry->{revision};
 		my $user = $logentry->{author};
 
-		my $when=time - str2time($logentry->{date}, 'UTC');
+		my $date = $logentry->{date};
+		$date =~ s/T/ /;
+		$date =~ s/\.\d+Z$//;
+		my $when=concise(ago(time - str2time($date, 'UTC')));
 
 		foreach my $msgline (split(/\n/, $logentry->{msg})) {
-			push @message, { line => $msgline };
+			push @message, { line => escapeHTML($msgline) };
 		}
 
 		my $committype="web";
 		if (defined $message[0] &&
-		    $message[0]->{line}=~/$config{web_commit_regexp}/) {
-			$user=defined $2 ? "$2" : "$3";
-			$message[0]->{line}=$4;
+		    $message[0]->{line}=~/$svn_webcommit/) {
+			$user="$1";
+			$message[0]->{line}=$2;
 		}
 		else {
 			$committype="svn";
@@ -184,12 +145,12 @@ sub rcs_recentchanges ($) { #{{{
 			$diffurl=~s/\[\[r1\]\]/$rev - 1/eg;
 			$diffurl=~s/\[\[r2\]\]/$rev/g;
 			push @pages, {
-				page => pagename($file),
+				link => htmllink("", "", pagename($file), 1),
 				diffurl => $diffurl,
 			} if length $file;
 		}
 		push @ret, { rev => $rev,
-			user => $user,
+			user => htmllink("", "", $user, 1),
 			committype => $committype,
 			when => $when,
 			message => [@message],
@@ -209,11 +170,10 @@ sub rcs_notify () { #{{{
 	
 	my $user=`svnlook author $config{svnrepo} -r $rev`;
 	chomp $user;
-	
 	my $message=`svnlook log $config{svnrepo} -r $rev`;
-	if ($message=~/$config{web_commit_regexp}/) {
-		$user=defined $2 ? "$2" : "$3";
-		$message=$4;
+	if ($message=~/$svn_webcommit/) {
+		$user="$1";
+		$message=$2;
 	}
 
 	my @changed_pages;
@@ -223,19 +183,48 @@ sub rcs_notify () { #{{{
 			push @changed_pages, $1;
 		}
 	}
-	
+		
 	require IkiWiki::UserInfo;
-	send_commit_mails(
-		sub {
-			return $message;
-		},
-		sub {
-			`svnlook diff $config{svnrepo} -r $rev --no-diff-deleted`;
-		}, $user, @changed_pages);
+	my @email_recipients=commit_notify_list($user, @changed_pages);
+	if (@email_recipients) {
+		# TODO: if a commit spans multiple pages, this will send
+		# subscribers a diff that might contain pages they did not
+		# sign up for. Should separate the diff per page and
+		# reassemble into one mail with just the pages subscribed to.
+		my $diff=`svnlook diff $config{svnrepo} -r $rev --no-diff-deleted`;
+
+		my $subject="$config{wikiname} update of ";
+		if (@changed_pages > 2) {
+			$subject.="$changed_pages[0] $changed_pages[1] etc";
+		}
+		else {
+			$subject.=join(" ", @changed_pages);
+		}
+		$subject.=" by $user";
+
+		my $template=template("notifymail.tmpl");
+		$template->param(
+			wikiname => $config{wikiname},
+			diff => $diff,
+			user => $user,
+			message => $message,
+		);
+		
+		eval q{use Mail::Sendmail};
+		foreach my $email (@email_recipients) {
+			sendmail(
+				To => $email,
+				From => "$config{wikiname} <$config{adminemail}>",
+				Subject => $subject,
+				Message => $template->output,
+			) or error("Failed to send update notification mail");
+		}
+	}
 } #}}}
 
 sub rcs_getctime ($) { #{{{
 	my $file=shift;
+	eval q{use Date::Parse};
 
 	my $svn_log_infoline=qr/^r\d+\s+\|\s+[^\s]+\s+\|\s+(\d+-\d+-\d+\s+\d+:\d+:\d+\s+[-+]?\d+).*/;
 		
@@ -257,8 +246,6 @@ sub rcs_getctime ($) { #{{{
 		return 0;
 	}
 		
-	eval q{use Date::Parse};
-	error($@) if $@;
 	$date=str2time($date);
 	debug("found ctime ".localtime($date)." for $file");
 	return $date;
