@@ -28,7 +28,7 @@ sub gen_wrapper () { #{{{
 	my @envsave;
 	push @envsave, qw{REMOTE_ADDR QUERY_STRING REQUEST_METHOD REQUEST_URI
 	               CONTENT_TYPE CONTENT_LENGTH GATEWAY_INTERFACE
-		       HTTP_COOKIE REMOTE_USER} if $config{cgi};
+		       HTTP_COOKIE} if $config{cgi};
 	my $envsave="";
 	foreach my $var (@envsave) {
 		$envsave.=<<"EOF"
@@ -46,12 +46,6 @@ EOF
 		asprintf(&newenviron[i++], "%s=%s", "REV", s);
 EOF
 	}
-	if ($config{rcs} eq "tla" && $config{notify}) {
-		$envsave.=<<"EOF"
-	if ((s=getenv("ARCH_VERSION")))
-		asprintf(&newenviron[i++], "%s=%s", "ARCH_VERSION", s);
-EOF
-	}
 	
 	$Data::Dumper::Indent=0; # no newlines
 	my $configstring=Data::Dumper->Dump([\%config], ['*config']);
@@ -64,7 +58,6 @@ EOF
 /* A wrapper for ikiwiki, can be safely made suid. */
 #define _GNU_SOURCE
 #include <stdio.h>
-#include <sys/types.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -82,11 +75,6 @@ $envsave
 	newenviron[i]=NULL;
 	environ=newenviron;
 
-	if (setregid(getegid(), -1) != 0 || setreuid(geteuid(), -1) != 0) {
-		perror("failed to drop real uid/gid");
-		exit(1);
-	}
-
 	execl("$this", "$this", NULL);
 	perror("failed to run $this");
 	exit(1);
